//Ohne Interrupts werden bei dieser Implementierung einfach
//In der Endlosschleife des Hauptprogramms stndig die Eingnge
//PA7 - entspricht Kanal A beim Drehgeber - und
//PD6 - entspricht Kanal Z beim Drehgeber - 
//beobachtet und die Dauern der Flankenwechsel gemessen.
//Fr Programmteile die nicht in die Zwischenzeiten passen,
//kann man sich behelfen, indem man nur zyklenweise
//die Geschwindigkeit mit.

//Fr die Zeitmessungen wird in dieser Klasse Timer2 als Zhler aktiviert.

//Grund dafr, keine Interrupts zu benutzen:
//Diese mssen fr die Funkbertragung frei bleiben.

class Drehgeber : public CoachBASIS, DrehgeberBASIS
{
    private:
        uint8_t  akku;
        uint8_t  akku2;
        uint8_t  anz;
        bool zustand;
        bool zustand2;
        int16_t vorwaerts;
	public:		
		void start(void)
		{
			DDRD  &= ~(1<<PD6);
			PORTD &= ~(1<<PD6); //kein Pullup
			DDRB  &= ~(1<<PB2);
			PORTB &= ~(1<<PB2); //kein Pullup


            //Timer 2 als Zhler mit Vorteilung von 256 definieren:

            //WGM20 = 0 WGM21 =0 => Zhlbetrieb, zhlt stndig von 0 bis 255
            //COM20 = 0 COM21 = 0 => Pin OC2 nicht verbunden
            //COM20 = 1 COM21 = 1 => Pin OC2 lschen on Top, setzen bei Compare
            //CS 22 21 20 : 000 gestoppt
            //CS 22 21 20 : 001 Vorteilung durch 1
            //CS 22 21 20 : 010 Vorteilung durch 8
            //CS 22 21 20 : 011 Vorteilung durch 32
            //CS 22 21 20 : 100 Vorteilung durch 64
            //CS 22 21 20 : 101 Vorteilung durch 128
            //CS 22 21 20 : 110 Vorteilung durch 256
            //CS 22 21 20 : 111 Vorteilung durch 1024
            TCCR2 = (1<<FOC2) | (0<<WGM20) | (0<<COM21) | (0<<COM20) | (0<<WGM21) | (1<<CS22) | (1<<CS21) | (1<<CS20);

            //Ein Zhlschritt in TCNT2 dauert dann
            //9216000/64 = 144000 Hz
            //144 Schritte laufen dann mit 1000Hz, dauern dann also eine Millisekunde.

            //Bei 200U/Min, 500 Pulsen pro Umdrehung ergibt sich am Drehgeberausgang
            //eine Maximale Pulsfrequenz von 1700Hz
            //Dann bentigt man eine Abtastung von mindestens 3400Hz
            //Mit einer Zhlergrenze von 36 lassen sich 4000Hz Abtastrate erzielen.

            //Die while-Schleife in holeOmegaViertel() liegt im Bereich von 100 Taktzyklen.
            //D.h. Ein Durchlauf kann 92160Hz erfolgen.
            //Selbst wenn sie 10mal langsamer wre, wre die Abtastrate in Ordnung.
		}

		void stop(void)
		{
            TCCR2 = (0<<FOC2) | (0<<WGM20) | (0<<COM21) | (0<<COM20) | (0<<WGM21) | (0<<CS22) | (0<<CS21) | (0<<CS20);
		}

        int holeOmegaViertel()
        {
            //Vorteilung: 1024
            //Wartezeit bei 36 Zhlschritten: 9/(9216000/1024) = 1ms
            //10ms entspricht 90 Zhlschritten
            //226 Zhlschritte entsprechen: 0,0251111s
            //Bei den verfgbaren 6V ergeben sich nur 60U/Min, da reicht diese Abtastung.
            //500 Pulse pro Umdrehung == 1000 Flankenwechsel pro Umdrehung
            //omega == rad/Sekunde
            //1 Flankenwechsel == 2PI/1000 rad
            //omega = Anzahl_Flankenwechsel*2PI/(1000*Dauer)
            //Hier:
            //omega = Anzahl_Flankenwechsel*2PI/(1000*0.0251111s)
            //omega_viertel ~= Anzahl_Flankenwechsel

            anz=0;
            TCNT2=0;
            vorwaerts=0;
            zustand=false;
            zustand2=false;
            while(true)
            {                
                akku = PIND;                        
                akku2 = PINB;                        
                if( (akku & (1<<PD6)) > 0  && !zustand )
                {
                    zustand=true;
                    if(zustand2==true)
                        vorwaerts--;
                    else
                        vorwaerts++;
                    anz++;
                }
                if( (akku & (1<<PD6)) == 0  && zustand )
                {
                    zustand=false;
                    anz++;
                }


                if( (akku2 & (1<<PB2)) > 0  && !zustand2 )
                    zustand2=true;
                if( (akku2 & (1<<PB2)) == 0  && zustand2 )
                    zustand2=false;

                if(TCNT2>225) //bei 226 break
                    break;
            }
            if(vorwaerts<0)
                return -anz;
            else
                return anz;
        }


};
