//Zuordnung der PORT-Bits zu dem Motortreiber:
//Mikrocontroller  Motortreiber
//OC1B / PD4       Enable 1
//PD7              In 1
//PD3              In 2
//Zuordnung der PORT-Bits zu dem Servo:
//OC1A / PD5       PWM-Eingang des Servo

//Da Motor und Servo ber Timer 1 miteinander verquickt sind,
//wird fr beides nur eine Klasse bereitgestellt, um
//unzulssige Zustnde bei einem von beidem zu vermeiden.

class MotorServo : public CoachBASIS, MotorServoBASIS
{
    private:
        int gwert_alt;
        int swert_alt;
    public:
        void start()
        {    
            gwert_alt = 9999;
            swert_alt = 9999;
            //Digitale Ausgnge nicht anrhren, da bereits in CoachBasis konfiguriert.
            TCCR1A = (1<<COM1A1) | (0<<COM1A0) | (1<<COM1B1) | (0<<COM1B0) | (0<<FOC1A) | (0<<FOC1B) | (1<<WGM11) | (1<<WGM10);
            TCCR1B = (0<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (0<<CS12) | (1<<CS11) | (1<<CS10);
            OCR1A=1023; //PWM wird invertiert bertragen, soll inaktiv sein
            OCR1B=1023; //PWM wird invertiert bertragen, soll inaktiv sein
            PORTD |= ((1<<PD3) | (1<<PD7)); //alles aus
        }

        void stop()
        {    
            PORTD |= ((1<<PD3) | (1<<PD7)); //alles aus
            OCR1A=1023;
            OCR1B=1023;
            TCCR1B = (0<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (0<<CS12) | (0<<CS11) | (0<<CS10);
            TCCR1A = (0<<COM1A1) | (0<<COM1A0) | (0<<COM1B1) | (0<<COM1B0) | (0<<FOC1A) | (0<<FOC1B) | (0<<WGM11) | (0<<WGM10);
        }

        void geschwindigkeit(int gwert) //+- 1023, +1023 vorwrts mit Maximalgeschw. , 1023 rckwrts. 
        {
            //Vermeidung unvorhersehbarer Zustnde:
            if(gwert>1023 || gwert<-1023)
                return;

            //Vermeidung berflssiger, ev. strender Aufrufe:
            if(gwert!=gwert_alt)
            {
                gwert_alt = gwert;

                PORTD |= ((1<<PD3) | (1<<PD7)); //alles aus
                if(gwert<0)
                {
                    gwert=-gwert;
                    PORTD &= ~(1<<PD7); //rckwrts
                }
                else
                {
                    PORTD &= ~(1<<PD3); //rckwrts
                }
                OCR1B=1023-gwert;
            }
	    }

        void richtung(int swert) ////-36..36 in 1,25Grad-Schritten, 0 Mitte, -36 links herum, +36 rechts herum
        {
            //Vermeidung unvorhersehbarer Zustnde:
            if(swert<-36 || swert>36)
                return;

            //Vermeidung berflssiger, ev. strender Aufrufe:
            if(swert!=swert_alt)
            {
                swert_alt = swert;

                //zwischen 72..144, aber invertiert
                OCR1A=915-swert;
            }
	    }

};
